----------------------------------------------------------------
-- Copyright (c) 2012 Klei Entertainment Inc.
-- All Rights Reserved.
-- SPY SOCIETY.
----------------------------------------------------------------

local util = include( "modules/util" )
local resources = include( "resources")
local modalDialog = include("states/state-modal-dialog")
local stateLoading = include( "states/state-loading" )
local mui = include( "mui/mui" )
local cdefs = include( "client_defs" )
local mui_defs = include( "mui/mui_defs" )
local mission_scoring = include("mission_scoring")
local mission_recap_screen = include ("hud/mission_recap_screen")
local metrics = include( "metrics" )
local ENDGAMEFLOW = mission_scoring.ENDGAMEFLOW
local death_dialog = include( "hud/death_dialog" )
local movieScreen = include('client/fe/moviescreen')
local serverdefs = include( "modules/serverdefs" )
local simdefs = include( "sim/simdefs" )
local stateMapScreen = include( "states/state-map-screen" )
local stateTeamPreview = include( "states/state-team-preview" )

----------------------------------------------------------------
local postgame = class()

----------------------------------------------------------------
function postgame:onLoad(sim, params, num_actions)
    log:write( "\tpostgame:onLoad()" )
    MOAIFmodDesigner.stopSound("alarm")
    MOAIFmodDesigner.playSound( "SpySociety/Music/stinger_victory")
    MOAIFmodDesigner.stopMusic()
    MOAIFmodDesigner.playSound("SpySociety/Music/music_map","theme")		
    FMODMixer:pushMix("frontend")

    self._sim = sim
        
    --save these because they get wiped when we finalize a losing game
    local user = savefiles.getCurrentGame()
    local campaign = user.data.saveSlots[ user.data.currentSaveSlot ]
    local oldcampaignDifficulty, olddifficultyOptions = campaign.campaignDifficulty, campaign.difficultyOptions

    --do our scoring stuff
    local flow_result  = mission_scoring.DoFinishMission(sim)
    metrics.level_finished():send( params, num_actions, sim )
    
    if flow_result == ENDGAMEFLOW.WON_CAMPAIGN then
        movieScreen("data/movies/end_Cinematic.ogv", function() self:OnFinalMovieDone( ) end)

    elseif flow_result == ENDGAMEFLOW.GAME_OVER then
        local screen = mission_recap_screen()
        screen:show(sim._resultTable,
            function()
                local rewardsDialog = death_dialog()
                rewardsDialog:show( false, function( option )

                    if option == 1 then

                        local user = savefiles.getCurrentGame()
                        local campaign = user.data.saveScumLevelSlots[ user.data.currentSaveSlot ]
                        if campaign then
                            campaign.hasShownInitialDeathTip = true
                            campaign.agency.missions_completed_1 = 0
                            campaign.agency.missions_completed_2 = 0
                            campaign.agency.missions_completed_3 = 0
                            sim:getStats():setStat( "missions_completed_1", 0 )
                            sim:getStats():setStat( "missions_completed_2", 0 )
                            sim:getStats():setStat( "missions_completed_3", 0 )

                            campaign.seed = campaign.seed + 1
                            campaign.uiMemento = nil
                            campaign.sim_history = nil
                        end

                        user.data.saveSlots[ user.data.currentSaveSlot ] = util.tcopy( campaign )
                        user:save()

                        local stateLoading = include( "states/state-loading" )
                        statemgr.deactivate( self )
                        stateLoading:loadCampaign( campaign )

                    elseif option == 2 then

                        local user = savefiles.getCurrentGame()
                        local campaign = user.data.saveScumDaySlots[ user.data.currentSaveSlot ]
                        if campaign then
                            campaign.hasShownInitialDeathTip = true
                            campaign.agency.missions_completed_1 = 0
                            campaign.agency.missions_completed_2 = 0
                            campaign.agency.missions_completed_3 = 0
                            sim:getStats():setStat( "missions_completed_1", 0 )
                            sim:getStats():setStat( "missions_completed_2", 0 )
                            sim:getStats():setStat( "missions_completed_3", 0 )

                            campaign.seed = campaign.seed + 1
                            campaign.uiMemento = nil
                            campaign.sim_history = nil
                        end

                        user.data.saveSlots[ user.data.currentSaveSlot ] = util.tcopy( campaign )
                        user:save()

                        local stateMapScreen = include( "states/state-map-screen" )
                        statemgr.activate( stateMapScreen(), campaign )
                        statemgr.deactivate( self )

                    else

                        --jcheng: :(
                        FMODMixer:popMix("nomusic")
                        FMODMixer:pushMix("frontend")
                        MOAIFmodDesigner.stopMusic()
                        MOAIFmodDesigner.stopSound("theme")

                        MOAIFmodDesigner.playSound( cdefs.SOUND_HUD_GAME_WOOSHOUT ) 
                        MOAIFmodDesigner.playSound("SpySociety/Music/music_title","theme")

                        local stateGenerationOptions = include( "states/state-generation-options" )
                        statemgr.activate( stateGenerationOptions(), oldcampaignDifficulty, olddifficultyOptions )
                        statemgr.deactivate( self )

                    end
                end)
            end )

    else
        local screen = mission_recap_screen()
        screen:show(sim._resultTable, function() self:OnFinishRecap( flow_result ) end )
        user:save()
    end
end


function postgame:onUnload()
    FMODMixer:popMix("frontend")
end

----------------------------------------------------------------
function postgame:onUpdate()

end


function postgame:OnFinalMovieDone()
    local rewardsDialog = death_dialog()
    
    local user = savefiles.getCurrentGame()
    local campaign = user.data.saveSlots[ user.data.currentSaveSlot ]
    local oldcampaignDifficulty, olddifficultyOptions = campaign.campaignDifficulty, campaign.difficultyOptions

    MOAIFmodDesigner.playSound("SpySociety/Music/music_map","theme")
    FMODMixer:pushMix("frontend")

    rewardsDialog:show( true, function()
        if oldcampaignDifficulty < simdefs.VERY_HARD_DIFFICULTY and oldcampaignDifficulty ~= -1 then
            oldcampaignDifficulty = oldcampaignDifficulty + 1
            olddifficultyOptions = nil
        end

        MOAIFmodDesigner.stopSound("theme")

        MOAIFmodDesigner.playSound( cdefs.SOUND_HUD_GAME_WOOSHOUT ) 
        MOAIFmodDesigner.playSound("SpySociety/Music/music_title","theme")

        local stateGenerationOptions = include( "states/state-generation-options" )
        statemgr.activate( stateGenerationOptions(), oldcampaignDifficulty, olddifficultyOptions )
        statemgr.deactivate( self )

    end)
end

function postgame:OnFinishRecap( flow_result )

    if flow_result == ENDGAMEFLOW.DONE_TUTORIAL then
        
        --go to the campaign generation screen
        local stateGenerationOptions = include( "states/state-generation-options" )
        statemgr.activate( stateGenerationOptions(), simdefs.NORMAL_DIFFICULTY )
        statemgr.deactivate( self )

        --movieScreen("data/movies/IntroCinematic.ogv", function() self:OnIntroMovieDone() end)
    elseif flow_result == ENDGAMEFLOW.CONTINUE_FINAL_MISSION then
        -- TODO: remove me.  don't think this happens for 1-level final mission.
        local user = savefiles.getCurrentGame()
        local campaign = user.data.saveSlots[ user.data.currentSaveSlot ]
        statemgr.deactivate( self )
        stateLoading:loadCampaign( campaign )
    
    elseif flow_result == ENDGAMEFLOW.CONTINUE_CAMPAIGN then
        local user = savefiles.getCurrentGame()
        local campaign = user.data.saveSlots[ user.data.currentSaveSlot ]
        local endless = campaign.difficultyOptions.maxHours == math.huge 
        statemgr.deactivate( self )
        stateLoading:loadUpgradeScreen( campaign.agency, endless, true )
    else
        assert(nil, "UNKNOWN LEVEL FINISH TYPE")
    end
end


return postgame
