----------------------------------------------------------------
-- Copyright (c) 2012 Klei Entertainment Inc.
-- All Rights Reserved.
-- SPY SOCIETY.
----------------------------------------------------------------

local util = include( "modules/util" )

----------------------------------------------------------------
-- Common definition tables re-used/extended elsewhere.

local HOTKEY_COLOUR = "<c:ffffff>"
local FLAVOUR_COLOUR = "<c:61AAAA>"
local ITEM_HEADER_COLOUR = "<ttheader>"
local DESC_COLOUR = "<c:ffffff>"
local EQUIPPED_COLOUR = "<c:FF8411>"
local NOTINSTALLED_COLOUR = "<c:FF8411>"
--local SPECIAL_INFO_COLOUR = "<c:F4FF78>"
local SPECIAL_INFO_COLOUR = "<c:ffffff>"

local function onAgentTooltip( tooltip, unit )
	local unitInfo = string.format( "" ) -- "HP: %d/%d", unit:getTraits().woundsMax - unit:getTraits().wounds, unit:getTraits().woundsMax 
	if unit:isPC() then
		tooltip:addLine( ITEM_HEADER_COLOUR..util.toupper(unit:getName()).."</>", unitInfo )
		if unit:isKO() then
			tooltip:addAbility( STRINGS.UI.TOOLTIP_AGENT_DOWN, STRINGS.UI.TOOLTIP_AGENT_DOWN_DESC, "gui/icons/item_icons/items_icon_small/icon-item_medigel_small.png" )
		end
	else
		if unit:isKO() then
			if unit:getTraits().paralyzed then
				unitInfo = string.format( STRINGS.UI.TOOLTIP_NAME_PARALYZED, unitInfo )
			else
				unitInfo = string.format( STRINGS.UI.TOOLTIP_NAME_KO_TIME, unitInfo, unit:getTraits().koTimer )
			end
		end
		tooltip:addLine( ITEM_HEADER_COLOUR..util.toupper(unit:getName()).."</>", unitInfo )

		if unit:getTraits().tagged then
			tooltip:addLine(  string.format( "<c:F4FF78>%s</>",STRINGS.UI.TOOLTIP_TAGGED) )
		end
	end
end

local function onItemWorldTooltip( tooltip, unit )
    tooltip:addLine( ITEM_HEADER_COLOUR..unit:getName().."</>" )

    if unit:getUnitData().flavor then
        tooltip:addDesc(  string.format("<c:61AAAA>"..unit:getUnitData().flavor).."</c>" )
    end	
    
    if unit:getUnitData().desc then
        tooltip:addDesc( unit:getUnitData().desc )
    end
end

local function onItemTooltip( tooltip, unit, userUnit )
    local simquery = include( "sim/simquery" )
    local name = util.toupper( unit:getName() )
	if unit:getTraits().ammo and unit:getTraits().maxAmmo then
		tooltip:addLine( ITEM_HEADER_COLOUR..name.."</>", string.format(STRINGS.ITEMS.AMMO,unit:getTraits().ammo,unit:getTraits().maxAmmo) )
	--elseif unit:getTraits().augment then
	--	tooltip:addLine( "<c:CCCC00>"..name.."</>" )
    else
		tooltip:addLine( ITEM_HEADER_COLOUR..name.."</>" )
	end

    if unit:getUnitData().flavor then
        tooltip:addDesc(  FLAVOUR_COLOUR..unit:getUnitData().flavor.."</>" )
    end	

   	if unit:getTraits().augment and unit:getTraits().installed == false then		
		tooltip:addLine( NOTINSTALLED_COLOUR.. STRINGS.UI.TOOLTIPS.NOT_INSTALLED .."</c>" )
	end

    if unit:getUnitData().desc then
        tooltip:addDesc( unit:getUnitData().desc )
    end

	if unit:getTraits().equipped then
		tooltip:addLine( EQUIPPED_COLOUR.. STRINGS.UI.TOOLTIPS.EQUIPPED .."</c>") 
	end
   	
	if unit:getUnitData().keyword then
		local keywordStrings = STRINGS.ITEMS.KEYWORDS[ unit:getUnitData().keyword ]
        if keywordStrings and keywordStrings.NAME and keywordStrings.DESC then
            tooltip:addAbility( keywordStrings.NAME, keywordStrings.DESC, "gui/items/icon-item_implant.png",nil,true )
        end
	end

	if unit:getTraits().cooldownMax then
		tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.RECHARGE, string.format(STRINGS.ITEMS.TOOLTIPS.RECHARGE_DESC, unit:getTraits().cooldownMax), "gui/icons/arrow_small.png" )
	end

	if not unit:getTraits().slot and unit:getTraits().damage then
		tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.KO_DAMAGE, string.format(STRINGS.ITEMS.TOOLTIPS.KO_DAMAGE_DESC, unit:getTraits().damage), "gui/icons/arrow_small.png" )
	end

	if unit:getTraits().range then
		tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.RANGE, string.format(STRINGS.ITEMS.TOOLTIPS.RANGE_DESC, unit:getTraits().range), "gui/icons/arrow_small.png" )
	end

	if unit:getTraits().disposable then
		tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.DISPOSABLE, STRINGS.ITEMS.TOOLTIPS.DISPOSABLE_DESC, "gui/icons/arrow_small.png" )
	end

	if unit:getTraits().cashInReward then
		tooltip:addLine( string.format( SPECIAL_INFO_COLOUR.. STRINGS.UI.TOOLTIPS.RETURN_FOR_CREDIT .."</c>", unit:getTraits().cashInReward) )
	end


	if unit:getTraits().pwrCost then
		tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.COSTPOWER, string.format(STRINGS.ITEMS.TOOLTIPS.COSTPOWER_DESC, unit:getTraits().pwrCost), "gui/icons/arrow_small.png" )
	end	

	if unit:getTraits().augment then
		if unit:getTraits().stackable then
			tooltip:addDesc( SPECIAL_INFO_COLOUR.. STRINGS.UI.TOOLTIPS.STACKABLE .."</>")
		else
			tooltip:addDesc( SPECIAL_INFO_COLOUR.. STRINGS.UI.TOOLTIPS.NOT_STACKABLE .."</>")
		end
	end

    local canUseAnyItem = false

    if userUnit and userUnit.agentDef then --and unit.agentDef.traits.useAnyItem 
    	for i,v in ipairs(userUnit.agentDef.upgrades) do 
    		if v.upgradeName == "augment_tony_2" then
    			canUseAnyItem	= true
    		end
    	end
    elseif userUnit and userUnit:getTraits() and userUnit:getTraits().useAnyItem then 
		canUseAnyItem= true
	end


	if unit:getRequirements() and userUnit then
		for skill,level in pairs( unit:getRequirements() ) do
            if not userUnit:hasSkill( skill, level ) and not canUseAnyItem then

            	local skilldefs = include( "sim/skilldefs" )
            	local skillDef = skilldefs.lookupSkill( skill )            	
			    tooltip:addRequirement( string.format( STRINGS.UI.TOOLTIP_REQUIRES_SKILL_LVL, util.toupper(skillDef.name), level ))
            end
        end
	end

    if unit:getTraits().installed and unit:getTraits().installed == false then		
		tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.INSTALL, STRINGS.ITEMS.TOOLTIPS.INSTALL_DESC, "gui/icons/arrow_small.png" )
	end

	
end


local corpse_template =
{
	type = "simunit",
	name = "Corpse",
	onWorldTooltip = function( tooltip, unit )
		tooltip:addLine( unit:getName() )
		if unit:getInventoryCount() > 0 then
			tooltip:addAbility( STRINGS.UI.ACTIONS.LOOT_CORPSE.NAME, STRINGS.UI.ACTIONS.LOOT_CORPSE.TOOLTIP, "gui/icons/arrow_small.png" )
		end
	end,
	kanim = "", -- Generated dynamically

	traits = { iscorpse=true, sightable=true, hidesInCover=true, interestSource=true },

	rig = "corpserig",
}

local basic_agent_traits =
{
	isAgent = true,
	hasSight = true, sightable = true,
	dynamicImpass = true,
	hasHearing = true,
	apMax = 1, 
	mpMax=8,
	mp=8,	
	ap = 1,
	silencer = true,
	meleeDamage = 0,
	maxThrow = 10,

	inventoryMaxSize = 3,
	augmentMaxSize = 0,

	skillsInjected = false,  

	corpseTemplate = corpse_template,
	
	canBeShot = true,
	canBeCritical = true,
	canKO = true,

	baseDamage = 0, 

	wounds = 0,	
	woundsMax = 1,

	hits = "blood",
	
	dashSoundRange = 5,
 
	isAiming=false, 
	selectpriority = 1,	
	sneaking = false,

	dragCostMod = 0,

}

local basic_guard_traits = util.extend( basic_agent_traits )
{
	canBeCritical = false,
	woundsMax = 1, 
	isGuard = true,
    cleanup = true, 	
	cashOnHand = 80,
	LOSrange = 8,
	LOSarc = math.pi / 4,
	LOSperipheralRange = 10,
	LOSperipheralArc = math.pi / 2,
	closedoors = true,

	patrolObserved = nil, 
	observablePatrol = true,
	noLoopOverwatch = true,
}

local basic_robot_traits =
{
	isAgent = true,
	isDrone = true,
	isMetal = true,
	sightable = true,
	hasSight = true,
	hasHearing = false,
	dynamicImpass = true,
	isGuard = true,
	noDoorAnim = true, 

	LOSrange = 8,
	LOSarc = math.pi / 2,

	notDraggable = true, 

	patrolObserved = nil, 
	observablePatrol = true,

	apMax = 1, 
	mpMax= 6,
	mp= 6,
	ap = 1, 
	wounds = 0,	
	woundsMax = 1,
	meleeDamage = 3,
	dashSoundRange = 6,

	mainframe_item = true,
    mainframe_ice = 1,
    mainframe_iceMax = 1,
	mainframe_status = "active",
    mainframe_no_recapture = true,
	
    corpseTemplate = corpse_template,
	
	canBeShot = true,
    canBeFriendlyShot = true,
	canBeCritical = false,
	canKO = false,
    empKO = 4, -- 4 ticks KO when EMP'd.
	
	baseDamage = 0, 

	controlTimer = 0, 
	controlTimerMax = 1, 

	hits = "spark",
	
	thoughtVis = nil,
 
	cleanup = true, 
	isAiming=false, 
	selectpriority = 1,	
	sneaking = false,

	walk=true,
	enforcer = false,
	noLoopOverwatch = true,
}

return 
{
	corpse_template = corpse_template,
	basic_agent_traits = basic_agent_traits,
	basic_guard_traits = basic_guard_traits,
	basic_robot_traits = basic_robot_traits,
	onAgentTooltip = onAgentTooltip,
    onItemTooltip = onItemTooltip,
    onItemWorldTooltip = onItemWorldTooltip,
}
	
