local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local commondefs = include("sim/unitdefs/commondefs")
local itemdefs = include("sim/unitdefs/itemdefs")
local speechdefs = include( "sim/speechdefs" )

---------------------------------------------------------------------------------------------------------
-- NPC templates

local SOUNDS = {
	GUARD = { 		appeared="SpySociety/HUD/gameplay/peek_negative", 
					alert ="SpySociety/Actions/guard/guard_alerted", 
					speech="SpySociety/Agents/dialogue_KO", 
					stealthstep = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL, 
					step = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL,

					getup = "SpySociety/Movement/foley_guard/getup",
					fall = "SpySociety/Movement/foley_guard/fall",
					fall_knee = "SpySociety/Movement/bodyfall_agent_knee_hardwood",
					fall_kneeframe = 9,
					fall_hand = "SpySociety/Movement/bodyfall_agent_hand_hardwood",
					fall_handframe = 20,
					land = "SpySociety/Movement/deathfall_agent_hardwood",
					land_frame = 34,					
					grabbed = "SpySociety/Movement/foley_guard/grabbed",
					pin = "SpySociety/Movement/foley_guard/pin_guard",
					pinned = "SpySociety/Movement/foley_guard/pinned",
					move ="SpySociety/Movement/foley_guard/move",
					hit = "SpySociety/HitResponse/hitby_ballistic_flesh",	
					
					die = "die",
					hurt_small = "SpySociety/Agents/<voice>/hurt_small",	
					hurt_large = "SpySociety/Agents/<voice>/hurt_large",	
				},
	ARMORED = { 	appeared="SpySociety/HUD/gameplay/peek_negative",  
					alert ="SpySociety/Actions/guard/guard_alerted", 
					speech="SpySociety/Agents/dialogue_KO" , 
					stealthstep = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL, 
					step = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL,

					getup = "SpySociety/Movement/foley_armoured/getup",
					fall = "SpySociety/Movement/foley_armoured/fall",
					fall_knee = "SpySociety/Movement/bodyfall_agent_knee_hardwood",
					fall_kneeframe = 9,
					fall_hand = "SpySociety/Movement/bodyfall_agent_hand_hardwood",
					fall_handframe = 20,	
					land = "SpySociety/Movement/deathfall_agent_hardwood",
					land_frame = 34,					
					grabbed = "SpySociety/Movement/foley_armoured/grabbed",
					pin = "SpySociety/Movement/foley_armoured/pin_guard",
					pinned = "SpySociety/Movement/foley_armoured/pinned",
					move ="SpySociety/Movement/foley_armoured/move",
					hit = "SpySociety/HitResponse/hitby_ballistic_flesh",

					die = "die",
					hurt_small = "SpySociety/Agents/<voice>/hurt_small",	
					hurt_large = "SpySociety/Agents/<voice>/hurt_large",	
				},
	HEAVY = { 		appeared="SpySociety/HUD/gameplay/peek_negative", 
					alert ="SpySociety/Actions/guard/guard_alerted", 
					speech="SpySociety/Agents/dialogue_KO", 
					stealthstep = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL, 
					step = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL,

					getup = "SpySociety/Movement/foley_heavy/getup",
					fall = "SpySociety/Movement/foley_heavy/fall",
					fall_knee = "SpySociety/Movement/bodyfall_agent_knee_hardwood",
					fall_kneeframe = 9,
					fall_hand = "SpySociety/Movement/bodyfall_agent_hand_hardwood",
					fall_handframe = 20,
					land = "SpySociety/Movement/deathfall_agent_hardwood",
					land_frame = 34,
					grabbed = "SpySociety/Movement/foley_heavy/grabbed",
					pin = "SpySociety/Movement/foley_heavy/pin_guard",
					pinned = "SpySociety/Movement/foley_heavy/pinned",
					move ="SpySociety/Movement/foley_heavy/move",
					hit = "SpySociety/HitResponse/hitby_ballistic_flesh",

					die = "die",
					hurt_small = "SpySociety/Agents/<voice>/hurt_small",	
					hurt_large = "SpySociety/Agents/<voice>/hurt_large",	
				},
	DRONE = { 		appeared="SpySociety/HUD/gameplay/peek_negative", 
					
					alert ="SpySociety/Actions/guard/guard_alerted", 
					--stealthstep = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL, 
					--step = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL,
					move_loop = "SpySociety/Objects/drone/drone_move_LP",
					reboot_end="SpySociety/Agents/Drone/Agitated/Wakeup",
					scan="SpySociety/Actions/Engineer/wireless_emitter",
					hit = "SpySociety/HitResponse/hitby_ballistic_metal",
					explode2 = "SpySociety/Objects/drone/drone_shutdown",
					explode2_frame =1,
				},		
	DRONE_WALK = { 	appeared="SpySociety/HUD/gameplay/peek_negative", 
					alert ="SpySociety/Actions/guard/guard_alerted", 
					stealthstep = simdefs.SOUNDPATH_FOOTSTEP_DRONE_HARDWOOD_NORMAL, 
					step = simdefs.SOUNDPATH_FOOTSTEP_DRONE_HARDWOOD_NORMAL,
					getup = "SpySociety/Movement/foley_walkingdrone/getup",
					fall = "SpySociety/Movement/foley_walkingdrone/fall",
					land = "SpySociety/Objects/drone/drone_hitground",
					land_frame = 9,					
					grabbed = "SpySociety/Movement/foley_walkingdrone/grabbed",
					pin = "SpySociety/Movement/foley_walkingdrone/pin_guard",
					pinned = "SpySociety/Movement/foley_walkingdrone/pinned",
					move ="SpySociety/Movement/foley_walkingdrone/move",
					reboot_end="SpySociety/Agents/Drone/Agitated/Wakeup",					
					scan="SpySociety/Actions/Engineer/wireless_emitter",
					hit = "SpySociety/HitResponse/hitby_ballistic_metal",
					explode = "SpySociety/Objects/drone/drone_explo",					
					explode_frame = 1,
					open = "SpySociety/Objects/drone/drone_open",
					open_frame = 12,
					close = "SpySociety/Objects/drone/drone_close",
					close_frame = 10,
					walk_pre = "SpySociety/Objects/drone/drone_close",
					walk_pst = "SpySociety/Objects/drone/drone_open",
					run_pst = "SpySociety/Objects/drone/drone_open",
					snk_pst = "SpySociety/Objects/drone/drone_open",

				},	
	DRONE_HOVER = { appeared="SpySociety/HUD/gameplay/peek_negative", 
					getko = "SpySociety/Objects/drone/drone_sleep",
					alert ="SpySociety/Actions/guard/guard_alerted", 
					--stealthstep = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL, 
					--step = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL,
					spot = "SpySociety/Objects/drone/drone_hover_LP",	

					move_loop_param = "move",		
					fall = "SpySociety/Objects/drone/drone_shutdown",
					land = "SpySociety/Objects/drone/drone_hitground",
					land_frame = 13,
					drop = "SpySociety/Objects/drone/drone_hitground",
					drop_frame = 11,	
					reboot_end="SpySociety/Agents/Drone/Agitated/Wakeup",						
					scan="SpySociety/Actions/Engineer/wireless_emitter",
					hit = "SpySociety/HitResponse/hitby_ballistic_metal",
					explode = "SpySociety/Objects/drone/drone_explo",
					explode_frame = 1,					
				},
	DRONE_HOVER_HEAVY = { appeared="SpySociety/HUD/gameplay/peek_negative", 
					getko = "SpySociety/Objects/drone/drone_sleep",
					
					alert ="SpySociety/Actions/guard/guard_alerted", 
					--stealthstep = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL, 
					--step = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL,
					spot = "SpySociety/Objects/drone/drone_hover_large_LP",	
					move_loop_param = "move",
					fall = "SpySociety/Objects/drone/drone_shutdown",
					land = "SpySociety/Objects/drone/drone_hitground",
					land_frame = 24,
					drop = "SpySociety/Objects/drone/drone_hitground",
					drop_frame = 15,					
					burst={{sound="SpySociety/Weapons/Precise/shoot_turret",soundFrames=1},{sound="SpySociety/Weapons/Precise/shoot_turret",soundFrames=5},{sound="SpySociety/Weapons/Precise/shoot_turret",soundFrames=6},{sound="SpySociety/Weapons/Precise/shoot_turret",soundFrames=7},{sound="SpySociety/Weapons/Precise/shoot_turret",soundFrames=8}},
					overwatch_pre = "SpySociety/Objects/drone/drone_gun_large_open",
					overwatch_pst = "SpySociety/Objects/drone/drone_gun_large_close",
					reboot_end="SpySociety/Agents/Drone/Agitated/Wakeup",		
					scan="SpySociety/Actions/Engineer/wireless_emitter",
					hit = "SpySociety/HitResponse/hitby_ballistic_metal",
					explode = "SpySociety/Objects/drone/drone_explo",
					explode_frame = 1,					
				},
}

local DEFAULT_IDLES =
{
	{
		pre="smoke_pre", idle="smoke_idle", action="smoke", pst="smoke_pst",
		sounds={{event="inhale", sound="SpySociety/Actions/guard/smoke_inhale"}, {event="exhale", sound="SpySociety/Actions/guard/smoke_exhale"}},
		idleCounts={min=8, max=15}
	},
	{
		pre="phone_pre", idle="phone_idle", action="phone", pst="phone_pst",
		sounds={{event="buzz", sound="SpySociety/Actions/guard/play_with_phone"}},
		idleCounts={min=8, max=15}
	},
}
local DEFAULT_ABILITIES = { "shootOverwatch", "overwatch", "breakDoor" }

local function onGuardTooltip( tooltip, unit )
	commondefs.onAgentTooltip( tooltip, unit )

    local traits = unit:getTraits()

    if traits.isSupportGuard then 
    	tooltip:addAbility( STRINGS.UI.TOOLTIPS.GRENADE_GUARD, STRINGS.UI.TOOLTIPS.GRENADE_GUARD_DESC,  "gui/icons/arrow_small.png" )
    end 

    if traits.mainframe_suppress_rangeMax then
		tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.NULL_ZONE ), util.sformat( STRINGS.UI.TOOLTIPS.NULL_ZONE_DESC, traits.mainframe_suppress_rangeMax ), "gui/icons/arrow_small.png" )
    end     

    if traits.isDrone then
	    tooltip:addAbility( STRINGS.UI.TOOLTIPS.CONTROLLABLE, STRINGS.UI.TOOLTIPS.CONTROLLABLE_DESC,  "gui/icons/arrow_small.png"  )				
    end

    if not traits.hasHearing then
	    tooltip:addAbility( STRINGS.UI.TOOLTIPS.NO_HEARING, STRINGS.UI.TOOLTIPS.NO_HEARING_DESC,  "gui/icons/arrow_small.png"  )				
    end
    if not traits.canKO then
	    tooltip:addAbility( STRINGS.UI.TOOLTIPS.KO_IMMUNE, STRINGS.UI.TOOLTIPS.KO_IMMUNE_DESC,  "gui/icons/arrow_small.png"  )									
    end
    if traits.heartMonitor == "enabled" then
        tooltip:addAbility( STRINGS.UI.TOOLTIPS.HEART_MONITOR, STRINGS.UI.TOOLTIPS.HEART_MONITOR_DESC, "gui/icons/item_icons/items_icon_small/icon-item_heart_monitor_small.png" )		
    end
    if traits.shieldArmor then
		tooltip:addAbility( STRINGS.UI.TOOLTIPS.BARRIER, STRINGS.UI.TOOLTIPS.BARRIER_DESC, "gui/icons/item_icons/items_icon_small/icon-item_personal_shield_small.png" )
    elseif traits.armor and traits.armor > 0 then
        tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.HEAVY_ARMOR, traits.armor ), STRINGS.UI.TOOLTIPS.HEAVY_ARMOR_DESC,  "gui/hud3/hud3_armor_tutorial_icon.png"  )
    end
    if traits.resistKO then
        tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.KO_RESISTANT, traits.resistKO ), STRINGS.UI.TOOLTIPS.KO_RESISTANT_DESC,  "gui/icons/arrow_small.png"  )
    end
    if (traits.LOSarc or 0) > (math.pi / 2) then
    	if traits.LOSarc <= math.pi then 
        	tooltip:addAbility( STRINGS.UI.TOOLTIPS.PERIPHERAL_BOOST, STRINGS.UI.TOOLTIPS.PERIPHERAL_BOOST_DESC,  "gui/icons/arrow_small.png"  )
        end 
    end
    if traits.neutralize_shield then
		tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.PROTECTIVE_SHIELDS, traits.neutralize_shield),  STRINGS.UI.TOOLTIPS.PROTECTIVE_SHIELDS_DESC, "gui/icons/item_icons/items_icon_small/icon-item_personal_shield_small.png",nil,true )
    end
	if traits.empDeath then
		tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.EMP_VULNERABLE ), STRINGS.UI.TOOLTIPS.EMP_VULNERABLE_DESC, "gui/icons/arrow_small.png",nil,true )
    end 

    if traits.mainframeRecapture then
		tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.COUNTER_HACK ), string.format( STRINGS.UI.TOOLTIPS.COUNTER_HACK_DESC, traits.mainframeRecapture), "gui/icons/arrow_small.png" )
    end 
    if traits.koDaemon then
		tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.KO_DEAMON ), string.format( STRINGS.UI.TOOLTIPS.KO_DEAMON_DESC), "gui/icons/arrow_small.png" )
    end 

    if traits.lookaroundRange then
		tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.DRONE_SCAN ), string.format( STRINGS.UI.TOOLTIPS.DRONE_SCAN_DESC), "gui/icons/arrow_small.png" )
    end     

    if traits.magnetic_reinforcement then
		tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.MAGNETIC_REINFOREMENTS ), string.format( STRINGS.UI.TOOLTIPS.MAGNETIC_REINFOREMENTS_DESC), "gui/icons/arrow_small.png" )
    end     

    local abilities = unit:getAbilities()

    for i, ability in ipairs( abilities ) do 
    	if ability.buffAbility then 
    		tooltip:addAbility( string.format( ability.name ), string.format( ability.buffDesc ), "gui/icons/arrow_small.png" )
    	end 
    end   
end

local DEFAULT_DRONE = 
{
	type = "simdrone",
	profile_icon = "gui/profile_icons/character-head-drone.png",
	onWorldTooltip = onGuardTooltip,
	kanim = "kanim_drone_SA",
	rig = "dronerig",
	traits =  util.extend( commondefs.basic_robot_traits )
	{
		mainframe_no_daemon_spawn = true,
		PWROnHand = 2,
		lookaroundRange = 4,
	},   

	voices = {"Drone"},
	speech = speechdefs.NPC,
	skills = {},
	abilities = {  "shootOverwatch", "overwatch" },
	children = {itemdefs.item_drone_turret},	
	brain = "DroneBrain",
}


local npc_templates =
{
	npc_business_man =
	{
		type = "simunit",
		name = STRINGS.GUARDS.CEO,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/executive_build",	
		profile_image = "executive.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_business_man",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			cashOnHand = 450, 
			ko_trigger = "intimidate_guard",
			kill_trigger = "guard_dead",
			vip=true,
            pacifist = true,
		},
		dropTable =
		{
			{ "item_adrenaline" ,5},
			{nil,75}
		},
		anarchyDropTable =
		{
			{ "item_laptop",5},
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_adrenaline" ,35},
			{nil,150}
		},		
		speech = speechdefs.NPC,
		voices = {"Executive"},
		skills = {},
		abilities = { },
		children = { },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "WimpBrain",		
	},

	npc_guard =
	{
		type = "simunit",
		name = STRINGS.GUARDS.GUARD,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/portrait_security_build",	
		profile_image = "portrait_security.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_male",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8
		},
		dropTable =
		{			
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{nil,90}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{nil,150}
		},			
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_pistol },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",		
	},

	ftm_guard =
	{
		type = "simunit",
		name = STRINGS.GUARDS.GUARD,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/portrait_security_build",
		profile_image = "portrait_security.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_male_ftm",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{nil,90}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{nil,150}
		},			
					
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_pistol },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",		
		blurb = "<ttheader>EMPLOYED</>\nIt is corporate policy to invest in life insurance for each an every one of it's valued employees. In event of accident causing injury or permanent system failure, dependents will receive insurance benefits in an order as noted by form CSF_312.\n<ttrule>Read all fine Print</>\n",
	},


	ftm_grenade_guard =
	{
		type = "simunit",
		name = STRINGS.GUARDS.SUPPORT_GUARD,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ftm_grenadier_build",
		profile_image = "FTM_grenadier.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_grenade_ftm",

		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{ "item_flashgrenade" ,50},
			{nil,90}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_flashgrenade" ,50},	    
			{ "item_adrenaline" ,15},
			{nil,150}
		},			
					
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_pistol, itemdefs.item_npc_flashgrenade },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",		
		blurb = "<ttheader>EMPLOYED</>\nIt is corporate policy to invest in life insurance for each an every one of it's valued employees. In event of accident causing injury or permanent system failure, dependents will receive insurance benefits in an order as noted by form CSF_312.\n<ttrule>Read all fine Print</>\n",
	},

	ftm_guard_tier_2 =
	{
		type = "simunit",
		name = STRINGS.GUARDS.ELITE_GUARD,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ftm_med_2_build",	
		profile_image = "FTM_med_2.png",

    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_tier2_male_ftm",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			armor = 1,	
			cashOnHand = 110, 

			LOSrange = 8,
			LOSarc = math.pi / 4,
			LOSperipheralRange = 10,
			LOSperipheralArc = math.pi / 2,
            lookaroundArc = math.pi / 2 + math.pi / 4,
            lookaroundOffset = math.pi / 4 - math.pi / 8
		},
		dropTable =
		{
			{ "item_emp_pack", 15 },
			{ "item_emp_pack_2", 6},
			{ "item_emp_pack_3", 4 },
			{ "item_shocktrap_2", 4 },
			{ "item_shocktrap_3", 4 },
			{nil, 68}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{ "item_emp_pack", 15 },
			{nil,150}
		},			
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_pistol2 },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "GuardBrain",
	},	


	barrier_guard =
	{
		type = "simunit",
		name = STRINGS.GUARDS.GUARD,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ftm_barrier_build",	
		profile_image = "FTM_barrier.png",

    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_barrier_guard_ftm",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,

			mainframe_item = true,
			mainframe_ice = 3,		
			mainframe_iceMax = 3,		
			mainframe_status = "active",	

			shieldArmor = true,
			firewallArmor = true,
			firewallShield = true,
			shields = 1,
			armor = 0,
			noTakeover = true,
			noRaiseIfIceZero = true,
			mainframe_ice_set_in_def = true,
			mainframe_no_daemon_spawn = true,
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{nil,90}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{ "item_emp_pack", 15 },
			{nil,150}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_pistol },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",		
	},

	sankaku_guard =
	{
		type = "simunit",
		name = STRINGS.GUARDS.GUARD,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/sankaku_med_build",
		profile_image = "sankaku_med.png",	
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_male_sankaku",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			cashOnHand = 110,
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{nil,90}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{nil,150}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_pistol },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",		
	},

	sankaku_guard_tier_2 =
	{
		type = "simunit",
		name = STRINGS.GUARDS.ELITE_GUARD,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/sankaku_med_2_build",
		profile_image = "sankaku_med_2.png",			
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_tier2_male_sankaku",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			armor = 1,	
			cashOnHand = 110, 

			LOSrange = 8,
			LOSarc = math.pi / 4,
			LOSperipheralRange = 10,
			LOSperipheralArc = math.pi / 2,					
            lookaroundArc = math.pi / 2 + math.pi / 4,
            lookaroundOffset = math.pi / 4 - math.pi / 8
		},
		dropTable =
		{
			{ "item_emp_pack", 15 },
			{ "item_emp_pack_2", 6},
			{ "item_emp_pack_3", 4 },
			{ "item_shocktrap_2", 4 },
			{ "item_shocktrap_3", 4 },
			{nil, 68}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{ "item_emp_pack", 15 },
			{nil,150}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_pistol2 },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "GuardBrain",
	},

	important_guard =
	{
		type = "simunit",
		name = STRINGS.GUARDS.CAPTAIN,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/enforcer2_build",	
		profile_image = "enforcer_2.png",
		profile_icon_36x36= "gui/profile_icons/security_36.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_male_enforcer_2",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			kill_trigger = "guard_dead",
			enforcer = false,
			dashSoundRange = 8, 
			sightable = true,
		},
		speech = speechdefs.NPC,
		voices = {"KO_Heavy"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_pistol },
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",		
	},

	plastek_guard =
	{
		type = "simunit",
		name = STRINGS.GUARDS.GUARD,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/plastech_med_build",	
		profile_image = "Plastech_med.png",	
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_male_plastek",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{nil,90}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{nil,150}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_pistol },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",
	},

	plastek_guard_tier2 =
	{
		type = "simunit",
		name = STRINGS.GUARDS.ELITE_GUARD,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/plastech_med_2_build",			
		profile_image = "Plastech_med_2.png",	
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_tier2_male_plastek",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			armor = 1,	
			cashOnHand = 90, 

			LOSrange = 8,
			LOSarc = math.pi / 4,
			LOSperipheralRange = 10,
			LOSperipheralArc = math.pi / 2,					
            lookaroundArc = math.pi / 2 + math.pi / 4,
            lookaroundOffset = math.pi / 4 - math.pi / 8
		},
		dropTable =
		{
			{ "item_clip" , 10},
			{ "item_laptop", 3 },
			{ "item_tazer_2" ,5},
			{ "item_tazer_3", 2 },
			{nil,80}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{ "item_emp_pack", 15 },
			{nil,135}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_pistol2 },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "GuardBrain",
	},

	plastek_recapture_guard =
	{
		type = "simunit",
		name = STRINGS.GUARDS.MKI,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/plastech_cloack_build",		
		profile_image = "Plastech_cloack.png",	
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ghost_male_plastek",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			mainframeRecapture = 6,	
			koDaemon = true,	

		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{nil,90}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{nil,150}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_pistol },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",
	},	


	plastek_firewall_guard =
	{
		type = "simunit",
		name = STRINGS.GUARDS.MKII,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/plastech_psi_build",	
		profile_image = "Plastech_psi.png",		
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_psi_male_plastek",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			firewallSupport = 2,
			firewallSupportRange = 8,
			investigateHackedDevices = true,
			armor = 1,	
			cashOnHand = 90, 

			LOSrange = 8,
			LOSarc = math.pi / 4,
			LOSperipheralRange = 10,
			LOSperipheralArc = math.pi / 2,					
            lookaroundArc = math.pi / 2 + math.pi / 4,
            lookaroundOffset = math.pi / 4 - math.pi / 8
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{nil,90}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{nil,150}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_pistol },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",
	},	


	ko_guard =
	{
		type = "simunit",
		name = STRINGS.GUARDS.GUARD,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ko_med_build",
		profile_image = "KO_med.png",	
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_male_ko",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	

		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{nil,90}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{nil,150}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		abilities = util.extend(DEFAULT_ABILITIES){},
		skills = {},
		children = { itemdefs.item_npc_pistol },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",
	},


	ko_grenade_guard =
	{
		type = "simunit",
		name = STRINGS.GUARDS.SUPPORT_GUARD,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ko_grenadier_build",
		profile_image = "KO_grenadier.png",	
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_grenade_ko",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{ "item_flashgrenade" ,50},
			{nil,90}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_flashgrenade" ,50},	    
			{ "item_adrenaline" ,15},
			{nil,150}
		},			
					
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_pistol, itemdefs.item_npc_flashgrenade },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",		
		blurb = "<ttheader>EMPLOYED</>\nIt is corporate policy to invest in life insurance for each an every one of it's valued employees. In event of accident causing injury or permanent system failure, dependents will receive insurance benefits in an order as noted by form CSF_312.\n<ttrule>Read all fine Print</>\n",
	},


	ko_guard_tier2 =
	{
		type = "simunit",
		name = STRINGS.GUARDS.ELITE_GUARD,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ko_med_2_build",
		profile_image = "KO_med_2.png",			
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_tier2_male_ko",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			armor = 1,		
			cashOnHand = 90, 

			LOSrange = 8,
			LOSarc = math.pi / 4,
			LOSperipheralRange = 10,
			LOSperipheralArc = math.pi / 2,					
            lookaroundArc = math.pi / 2 + math.pi / 4,
            lookaroundOffset = math.pi / 4 - math.pi / 8
		},
		dropTable =
		{
			{ "item_clip" , 8},
			{ "item_stim", 7},
			{ "item_stim_2", 3},
			{nil,79},
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{nil,135}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_pistol2 },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "GuardBrain",
	},

	ko_guard_heavy =
	{
		type = "simunit",
		name = STRINGS.GUARDS.HEAVY_GUARD,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ko_heavy_build",
		profile_image = "KO_heavy.png",					
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_male_ko_heavy",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			mpMax=6,
			mp=6,		
			wounds = 0,	
			armor = 2,
            resistKO = 1,
			backPowerPack = true,
			backPowerPackoverloadKO = 6,
			cashOnHand = 90,
		},
		dropTable =
		{
			{ "item_clip" , 8},
			{ "item_stim", 7},
			{ "item_stim_2", 6},
			{ "item_stim_3", 5 },
			{ nil, 74 },
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{nil,135}
		},		
		speech = speechdefs.NPC,
		voices = {"KO_Heavy",},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_smg },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.HEAVY,
		brain = "GuardBrain",
	},	

	ko_specops =
	{
		type = "simunit",
		name = STRINGS.GUARDS.SPEC_OPS,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ko_spec_ops_build",
		profile_image = "KO_spec_ops.png",		
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_specops_ko",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 6,	
			LOSarc = math.pi,
            lookaroundArc = math.pi + math.pi / 4,
            lookaroundOffset = math.pi / 8
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_stim_2", 6},
			{ "item_stim_3", 5 },
			{ "item_cloakingrig_3", 10 },
			{nil, 74},
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{nil,135}
		},		
		speech = speechdefs.NPC,
		voices = {"KO_SpecOps",},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_rifle },
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",
	},

	npc_guard_enforcer =
	{
		type = "simunit",
		name = STRINGS.GUARDS.ENFORCER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/enforcer_build",
		profile_image = "enforcer.png",	
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_male_enforcer",
		traits = util.extend( commondefs.basic_guard_traits )
		{
			heartMonitor="enabled",
			enforcer = true,
			dashSoundRange = 8,
		},
		speech = speechdefs.NPC,
		voices = {"KO_Heavy",},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_smg, itemdefs.item_npc_scangrenade},
		brain = "GuardBrain",
		sounds = SOUNDS.ARMORED,
	},
	
	npc_guard_enforcer_reinforcement =
	{
		type = "simunit",
		name = STRINGS.GUARDS.ENFORCER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/enforcer2_build",
		profile_image = "enforcer_2.png",		
		profile_icon_36x36= "gui/profile_icons/security_36.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_male_enforcer_2",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			heartMonitor="enabled",
			enforcer = true,
			dashSoundRange = 8, 
		},
		speech = speechdefs.NPC,
		voices = {"KO_Heavy"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_smg, itemdefs.item_npc_scangrenade},
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",		
	},


	npc_guard_enforcer_reinforcement_2 =
	{
		type = "simunit",
		name = STRINGS.GUARDS.ELITE_ENFORCER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/enforcer_build",
		profile_image = "enforcer.png",	
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_male_enforcer",
		traits = util.extend( commondefs.basic_guard_traits ) 
		{
			heartMonitor="enabled",
			enforcer = true,
			dashSoundRange = 8,
			armor = 1,
		},
		speech = speechdefs.NPC,
		voices = {"KO_Heavy",},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_smg, itemdefs.item_npc_scangrenade},
		brain = "GuardBrain",
		sounds = SOUNDS.ARMORED,
	},	
	
	drone = util.extend( DEFAULT_DRONE )
	{
		name = STRINGS.GUARDS.OBAKE_DRONE,
		profile_anim = "portraits/sankaku_drone_face_new",
		profile_build = "portraits/sankaku_drone_face_new",
		profile_image = "sankaku_drone.png",		
		kanim = "kanim_drone_SA",
		sounds = SOUNDS.DRONE_WALK,
	},

	drone_tier2 = util.extend( DEFAULT_DRONE )
	{
		name = STRINGS.GUARDS.OBAKE_DRONE2,
		profile_anim = "portraits/sankaku_drone_heavy_face_new",
		profile_build = "portraits/sankaku_drone_heavy_face_new",
		profile_image = "sankaku_drone_heavy.png",					
		kanim = "kanim_drone_tier2_SA",
		sounds = SOUNDS.DRONE_WALK,
    	onWorldTooltip = onGuardTooltip,
		traits = util.extend( DEFAULT_DRONE.traits )   
		{
			mainframe_ice = 2,	
			mainframe_iceMax = 2,
			armor = 1,
            lookaroundPeripheralArc = math.pi,
		},
	},

	null_drone = util.extend( DEFAULT_DRONE )
	{
		name = STRINGS.GUARDS.NULL_DRONE,
		kanim = "kanim_drone_null_SA",
		sounds = SOUNDS.DRONE_HOVER,
		profile_anim = "portraits/sankaku_drone_null_new",
		profile_image = "sankaku_drone_null.png",		
		brain = "PacifistBrain",
		abilities = { _OVERRIDE = true },
		children = { _OVERRIDE = true },
		traits = util.extend( DEFAULT_DRONE.traits )
		{
            mainframe_no_daemon_spawn = false,
			mainframe_always_daemon_spawn = true,
			mainframe_suppress_rangeMax = 4,
			mainframe_suppress_range = 4,
			mainframe_ice = 3,
			mainframe_iceMax = 3,

            dynamicImpass = false,
			scanSweeps = true,
            pacifist = true,
		},
		dropTable = 
		{
			{ "item_portabledrive", 25 },
			{ "item_portabledrive_2", 5 },
			{ "item_portabledrive_3", 3 },
			{nil, 67}
		},
	},

	camera_drone = util.extend( DEFAULT_DRONE )
	{
		type = "simcameradrone",
		name = STRINGS.GUARDS.CAMERA_DRONE,
		kanim = "kanim_drone_camera_SA",
		brain = "PacifistBrain",
		sounds = SOUNDS.DRONE_HOVER, 
		profile_anim = "portraits/sankaku_drone_camera_new",
		profile_image = "sankaku_drone_camera.png",		
		children = { _OVERRIDE = true },
		abilities = { _OVERRIDE = true },
		traits = util.extend( DEFAULT_DRONE.traits )
		{
			camera_drone = true, 
			controlTimerMax = 2, 
			dashSoundRange = 0,
            dynamicImpass = false,
            empDeath = true,

            scanSweeps = true,
            pacifist = true,
            PWROnHand = 1,
		},
		dropTable = 
		{
			{nil, 67}
		},
	},

	drone_akuma = 
	{
		type = "simdrone",
		name = STRINGS.GUARDS.AKUMA_DRONE,
		profile_icon = "gui/profile_icons/character-head-drone.png",
		profile_anim = "portraits/sankaku_drone_tank_face_new",
		profile_build = "portraits/sankaku_drone_tank_face_new",
		profile_image = "sankaku_drone_tank.png",		
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_drone_akuma_SA",
		rig = "dronerig",
		traits = util.extend( commondefs.basic_robot_traits )   
		{
			walk=true,
			enforcer = false,
			dashSoundRange = 8,		
			mpMax=4,
			mp=4,			
			
			mainframe_no_daemon_spawn = false,
			mainframe_always_daemon_spawn = true,
            mainframe_ice = 4,
            mainframe_iceMax = 4,
            empKO = 2,

			LOSarc = 3*math.pi/4,

			armor = 2,
			PWROnHand = 4,
            lookaroundPeripheralArc = math.pi,
		},

		voices = {"Drone"},
		speech = speechdefs.NPC,
		skills = {},
		dropTable = 
		{
			{ "item_icebreaker", 25 },
			{ "item_icebreaker_2", 5 },
			{ "item_icebreaker_3", 3 },
			{nil, 67}
		},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = {itemdefs.item_npc_smg_drone},
		sounds = SOUNDS.DRONE_HOVER_HEAVY,
		brain = "DroneBrain",
	},	

	omni_observer = 
	{
		type = "simunit",
		name = STRINGS.GUARDS.OMNI_OBSERVER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/sankaku_med_2_build",
		profile_image = "sankaku_med_2.png",					
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_tier2_male_sankaku",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			armor = 0,	
            seesHidden = true, 
            detect_cloak = true, 
            omni = true,
		},
		dropTable = 
		{
			{nil, 67}
		},
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = {"shootOverwatch", "overwatch", "breakDoor", "peripheral_expansion_passive", "ultrasonic_echolocation_passive", "ultraviolet_spectrometer_passive" },
		children = { itemdefs.item_npc_pistol2 },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "GuardBrain",
	},
	
	omni_hunter = 
	{
		type = "simunit",
		name = STRINGS.GUARDS.OMNI_HUNTER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/sankaku_med_2_build",
		profile_image = "sankaku_med_2.png",								
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_tier2_male_sankaku",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			mpMax = 6, 
			mp = 6, 
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			armor = 0,	
            omni = true,

			sensors = 1, 
			sensorsMax = 1, 
		},
		dropTable = 
		{
			{nil, 67}
		},
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = {"shootOverwatch", "overwatch", "breakDoor", "long_range_sensors_passive", "sprint_pads_passive", "regenerative_nanocells_passive" },
		children = { itemdefs.item_npc_pistol2 },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "GuardBrain",
	},

	omni_crier = 
	{
		type = "simunit",
		name = STRINGS.GUARDS.OMNI_CRIER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/omni_med_build",
		profile_image = "sankaku_med_2.png",								
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_omni_crier",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			armor = 1,	
            omni = true,
		},
		dropTable = 
		{
			{nil, 67}
		},
		beginnerTraits = 
		{
			armor = 0, 
		},
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = {"shootOverwatch", "overwatch", "breakDoor", "improved_heart_monitor_passive", "recon_protocol_passive", "consciousness_monitor_passive" },
		children = { itemdefs.item_npc_pistol2 },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "GuardBrain",
	},

	omni_killer = 
	{
		type = "simunit",
		name = STRINGS.GUARDS.OMNI_KILLER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/sankaku_med_2_build",
		profile_image = "sankaku_med_2.png",								
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_tier2_male_sankaku",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			armor = 0,	
            omni = true,

		},
		dropTable = 
		{
			{nil, 67}
		},
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = {"shootOverwatch", "overwatch", "breakDoor", "overtuned_reflexes_passive", },
		children = { itemdefs.item_npc_pistol2 },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "GuardBrain",
	},

	omni_protector = 
	{
		type = "simunit",
		name = STRINGS.GUARDS.OMNI_PROTECTOR,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/omni_barrier_build",
		profile_image = "sankaku_med_2.png",								
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_omni_barrier",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,

			mainframe_item = true,
			mainframe_ice = 8,
			mainframe_iceMax = 8,		
			mainframe_status = "active",	

			shieldArmor = true,
			firewallArmor = true,
			firewallShield = true,
			shields = 1,
			armor = 0,
			noTakeover = true,
			noRaiseIfIceZero = true,
			mainframe_ice_set_in_def = true,
			mainframe_no_daemon_spawn = true,

            omni = true,			
		},
		dropTable = 
		{
			{nil, 67}
		},
		beginnerTraits = 
		{
			mainframe_ice = 6, 
			mainframe_iceMax = 6, 
		},
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = {"shootOverwatch", "overwatch", "breakDoor", "mainframe_attunement_passive", },
		children = { itemdefs.item_npc_rifle },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "GuardBrain",
	},

	omni_soldier = 
	{
		type = "simunit",
		name = STRINGS.GUARDS.OMNI_SOLDIER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/omni_heavy_build",
		profile_image = "sankaku_med_2.png",								
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_omni_soldier",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			armor = 3,
			omniArmorGuard = true, 
            omni = true,
		},
		dropTable = 
		{
			{nil, 67}
		},
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = {"shootOverwatch", "overwatch", "breakDoor",  },
		children = { itemdefs.item_npc_smg_omni },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "GuardBrain",
	},

}

npc_templates.dummy_guard = util.extend( npc_templates.npc_guard )
{
	dropTable = nil
}


-- Reassign key name to value table.
for id, template in pairs(npc_templates) do
	template.id = id
end


return npc_templates
