----------------------------------------------------------------
-- Copyright (c) 2012 Klei Entertainment Inc.
-- All Rights Reserved.
-- SPY SOCIETY.
----------------------------------------------------------------

local util = include( "modules/util" )
local array = include( "modules/array" )
local simunit = include( "sim/simunit" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local simfactory = include( "sim/simfactory" )
local mathutil = include( "modules/mathutil" )

-----------------------------------------------------
-- Local functions

local emp = { ClassType = "simemppack" }

function emp:detonate( sim )
	local x0,y0 = self:getLocation()
    local units = self:getTargets( x0, y0 )

	sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = x0, y = y0, units = units, range = self:getTraits().range } )

    for i, unit in ipairs(units) do
        unit:processEMP( self:getTraits().emp_duration, true )
    end

    -- Destroy the EMP.
	sim:warpUnit( self, nil )
	sim:despawnUnit( self )
	sim:emitSound( simdefs.SOUND_SMALL, x0, y0, nil )
end

function emp:onWarp(sim)
	if self:getLocation() then
		sim:addTrigger( simdefs.TRG_END_TURN, self )
	else
		sim:removeTrigger( simdefs.TRG_END_TURN, self )		
	end
end 

function emp:onTrigger( sim, evType, evData )
	if evType == simdefs.TRG_END_TURN then
		if self:getTraits().primed then
			self:detonate( sim )	
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self} ) 		
		end
		
	end
end

function emp:getTargets( x0, y0 )
    local cells = simquery.rasterCircle( self._sim, x0, y0, self:getTraits().range )
    local units = {}
    for i, x, y in util.xypairs( cells ) do
        local cell = self._sim:getCell( x, y )
        if cell then
            for _, cellUnit in ipairs(cell.units) do
                if cellUnit ~= self and (cellUnit:getTraits().mainframe_status or cellUnit:getTraits().heartMonitor) then
                    table.insert( units, cellUnit )
                end
            end
		end
	end

    return units
end

-----------------------------------------------------
-- Interface functions

local function createEMPPack( unitData, sim )
	return simunit.createUnit( unitData, sim, emp )
end

simfactory.register( createEMPPack )

return
{
	createEMPPack = createEMPPack,
}

